function x = colsweep(L, b, bs);
%--------------------------------------------------------------------------------
% Can call this thingy with
%
% x = colsweep(L, b, bs);
%   or
% x = colsweep(L, b) 
%   or
% b = colsweep(L, b)   (The most common case)
%
% Solve Lx = b, where L is lower triangular. This is called a "column sweep" 
% version since it accesses a column of the matrix L in the innermost loop.
% The rhs b and x can be aliased. 
%
% Since bs is an unused optional argument, here to retain argument signature
% compatibility with block versions.
%
% The Boolean checklower = true specifies checking to make sure L is lower
% triangular by using the norm of the strictly upper triangular part of L. In
% practice, since the code does not reference it otherwise, that type of "error"
% check is not needed.
%--------------------------------------------------------------------------------


checklower = false; % Test for L being lower triangular or not
x = b;            % Needed if early return is made from errors

if isempty(L)
    disp('Error in colsweep; L should be nonempty')
    x = [];
    return
end

% Only intended to be used for square matrices
[m,n] = size(L);  % Get the size of L
if (m ~= n | length(b) ~= m)
    disp('Error in colsweep; dims of L and b must be conformal')
    return
end

if checklower % Make sure that L is lower triangular
    upper_part = triu(L,1);  % 1 means exclude the main diagonal of L
    up_norm = norm(upper_part,1);
    if (up_norm > n*eps)
        disp('Error in colsweep; L must be lower triangular')
        disp(['Upper part has one-norm of ', num2str(up_norm)]);
        return
    end
end

% Now for the actual computations:
for j = 1:n
    x(j) = x(j)/L(j,j);
    for i = j+1:n
        x(i) = x(i) - L(i,j)*x(j);
    end 
end 

