function x = rowsweep(L, b, bs);
%---------------------------------------------------------------------------------
% x = rowsweep(L,b,bs);
%   or
% x = rowsweep(L,b);
%   or
% x = rowsweep(L,x);
%
% Solve Lx = b, where L is lower triangular. This is called a "row sweep" version
% since it accesses a row of the matrix L in the innermost loop. The rhs b and x
% can be aliased.
%
% The Boolean parameter checklower specifies testing to make sure L has only 
% zeros in its strictly upper triangular part. That generally is not wanted
% or needed, and in any case the solve does not reference any of the strictly
% upper triangular entries in L.
% 
% The blocksize variable bs is not used, but is present to be compatible with
% the block version.
%
% This uses loops, while in practice Matlab would use vector statements, at
% least for the innermost loop. Actually, 
%       x = tril(L)\b
% will do the solve far faster than any set of loops.
%---------------------------------------------------------------------------------

checklower = false; % Test for L being lower triangular or not
x = b;  % In case of error, x must be defined on return

if isempty(L)
    disp('Error in rowsweep; matrix should be nonempty')
    x = [];
    return
end

[m,n] = size(L);
if (m~=n | length(b) ~= m)
    disp('Error in rowsweep; L must be square')
    return
end

if checklower % Make sure that L is lower triangular
    up_norm = norm(triu(L,1), 1);
    if (up_norm > n*eps)
        disp('Error in rowsweep; L must be lower triangular')
        disp(['Upper part has one-norm of ', num2str(up_norm)]);
        disp(sprintf('m = %d, n = %d \n', m, n))
        return
    end
end

% At last, the actual triangular solve:
for i = 1:n
     % What BLAS function does the next loop implement?
    for j = 1:i-1
        x(i) = x(i) - L(i,j)*x(j);
    end
    x(i) = x(i)/L(i,i);
end

