%--------------------------------------------------------------------------------
% Script to run versions of lower triangular solvers; rowsweep, colsweep, and
% their block variants are available. This demonstrates that even a simple lower
% triangular solve can have drastically bad solutions in some measure or another.
% Although the matrix called "nightmare" is famous for blowing up, really bad
% answers can occur for random values. To help illuminate this, the singular 
% values of L can be calculated and plotted. A unit lower triangular matrix has
% all eigenvalues = 1, but this is a good demonstration why eigenvalues are 
% complete crap for assuring numerical niceness.
%
% Another point is that the residual vector norm can be small even if the putative 
% solution vector is way off from the (allegedly) true solution vector.
%
%   n        = matrix order
%   bs       = blocksize; need not evenly divide n
%   whichwun = which of the four lower triangular solves to use
%
% Three kinds of lower triangular matrices are available (see setl.m)
%   random
%   unit lower
%   (strongly) diagonally dominant
% Making the matrix "diagonally dominant" means the absolute value of each
% diagonal entry T(i,i) is larger than the sum of the absolute values of the
% off-diagonal values. This makes the system more numerically stable.
%
% Two kinds of solution vectors are defined:
%   xtrue = where the right hand side vector b is defined as b = L*xtrue
%   x     = solution computed by one of the four triangular solve functions
%
% Error and residual norms can be checked and plotted. Because the components of
% those error vectors can be positive, zero, or negative and log-scaled plots 
% are used, their absolute values are plotted instead of their signed values.
% Matlab does The Right Thing if some component is exactly zero.
%
% The first two blobs of code are for user-settable parameters. 
%--------------------------------------------------------------------------------

% Matrix type and size(s)
n             = 90; 
bs            = 12;    % Not referenced if using method 1 or 2.
diagdom       = false; % Make L strongly diagonally dominant
unitL         = false; % Make L unit lower triangular
nightmare     = false; % Make L, well, a nightmare

% What to show
displaysvd    = false; % To see the singular values of L
displayres    = true;  % Plot the residual vector components
displayerr    = true;  % Plot the error vector components
displaytime   = true;  % Give timing


% Get user input to choose which solver to use
disp('Methods to choose from are numbered as')
disp('    1 = column sweep')
disp('    2 = row sweep')
disp('    3 = block column sweep')
disp('    4 = block row sweep')
whichwun = input('\nWhich of the four versions to use [1-4]: ');
if isempty(whichwun)
    whichwun = 4;
end

if (whichwun < 1 || whichwun > 4)
    disp('Are you insane? Just pick a method from the integers 1-4')
    disp('Setting the version to 4, since you are incompetent')
    whichwun = 4;
end

disp(sprintf('\nn = %d', n))
if (whichwun > 2)
    disp(sprintf('blocksize = %d', bs))
end

% Define L
L = setl(n,diagdom, unitL, nightmare);

% Set the a priori known vectors
xtrue = randn(n,1);  % the one true vector to rule them all
b     = L*xtrue;     % so xtrue is guaranteed to solve Lx = b
% Look at Matlab's solution as well
t0 = clock;
switch whichwun
    case{1}
        t0 = clock;
        x = colsweep(L,b);
        t1 = clock;
        s = sprintf('\ncolsweep() gives residual norm(L*x - b)  = %g', norm(L*x - b));
    case{2}
        t0 = clock;
        x = rowsweep(L,b);
        t1 = clock;
        s = sprintf('\nrowsweep gives residual norm(L*x - b)    = %g', norm(L*x - b));
    case{3}
        t0 = clock;
        x = bcolsweep(L,b,bs);
        t1 = clock;
        s = sprintf('\nbcolsweep gives residual norm(L*x - b)   = %g', norm(L*x - b));
    case{4}
        t0 = clock;
        x = browsweep(L,b,bs);
        t1 = clock;
        s = sprintf('\nbrowsweep gives residual norm(L*x - b)   = %g', norm(L*x - b));
    otherwise
        disp('Dood (or doodess), learn to read. Gimme whichwun as an integer in 1:4')
        return
end
disp(s)
disp(sprintf('Error norm(xtrue - x) = %g\n', norm(xtrue - x)));

if displaytime
    timetaken = etime(t1, t0);
    flops = n*n;
    mflops = flops/(timetaken*1.0e6);
    disp(['Elapsed time: '  num2str(timetaken) ' seconds'])
    disp(['Compute rate: '  num2str(mflops) ' Mflop/sec'])
end

if displayres
    positionfig;
    %r = b - L*x;
    semilogy([1:n], abs(b - L*x), 'b*')
    title('Absolute Value of Residual Vector abs(L*x - b)');
    xlabel('Component Number');
    ylabel('Residual');

    % Warn about any breaks in the plot line (log(0) = -inf)
    I = find(abs(L*x-b) <= realmin); 
    if ~isempty(I) 
        disp(sprintf('%d of the residual components are exactly zero', length(I)))
        disp(sprintf('This shows as breaks in the plot of the residual\n'))
    end
end

if displayerr
    positionfig;
    semilogy([1:n], abs(x - xtrue), 'm*')
    title('Absolute Value of Error Vector abs(x - xtrue)');
    xlabel('Component Number');
    ylabel('Absolute Error');
    
    I = find(abs(x-xtrue) <= realmin); 
    if ~isempty(I) 
        disp(sprintf('%d of the error vector components are exactly zero',...
            length(I)))
        disp(sprintf('This shows as breaks in the plot of the error vector\n'))
    end
end

% In case of blowup, helps to see why
if displaysvd
    s = svd(L);
    condL = max(s)/min(s);
    disp(sprintf('Condition number of L = %g', condL))
    positionfig;
    semilogy(1:n,s,'b+')
    ylabel('\sigma')
    xlabel('Singular Value Number')
    title('Singular Values of L')
end
